==> klayout-0.21.19.tar.gz: klayout-0.21.19/src/layFileDialog.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtGui/QFileDialog>
#include <QtCore/QFileInfo>

#include "layFileDialog.h"
#include "layMainWindow.h"
#include "tlInternational.h"

namespace lay
{

static bool s_use_utf8 = false;

//  HACK: see layMain.cc
void
FileDialog::set_utf8 (bool f)
{
  s_use_utf8 = f;
}

FileDialog::FileDialog (QWidget *parent, const std::string &title, const std::string &filters, const std::string &def_suffix)
  : QObject (parent)
{
  m_dir = QDir::current ();
  m_title = title.c_str ();
  m_filters = filters.c_str ();
  m_def_suffix = def_suffix.c_str ();
}

FileDialog::~FileDialog ()
{
  //  .. nothing yet ..
}

//  not used if standard dialogs are used (disabled to avoid compiler warning):
#if 0
static void
split_filters (const QString &filters, QStringList &slist)
{
  QString s;
  for (const char *cp = filters.ascii (); *cp; ++cp) {
    if (cp[0] == ';' && cp[1] == ';') {
      slist << s;
      ++cp;
      s = "";
    } else {
      s += *cp;
    }
  }

  if (s != "") {
    slist << s;
  }
}
#endif

bool 
FileDialog::get_open (std::string &fp) 
{
#if 1
  //  Use the standard (system) dialogs:

  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fp.c_str ();
  } else {
    file_name = m_dir.absolutePath ();
  }

  QString f = QFileDialog::getOpenFileName (lay::MainWindow::instance (), QString (tl::translate ("Load ").c_str ()) + m_title, file_name, m_filters);

  if (! f.isEmpty ()) {
    if (s_use_utf8) {
      fp = f.toUtf8 ().constData ();
    } else {
      fp = f.ascii ();
    }
    QFileInfo fi (f);
    m_dir = fi.absoluteDir ();
    return true;
  } else {
    return false;
  }
#else
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (true);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::ExistingFile);
  fdia.setAcceptMode (QFileDialog::AcceptOpen);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Open ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      if (s_use_utf8) {
        fp = files [0].toUtf8 ().constData ();
      } else {
        fp = files [0].ascii ();
      }
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
#endif
}

bool 
FileDialog::get_open (std::vector<std::string> &fp, const std::string &dir) 
{
#if 1
  //  Use the standard (system) dialogs:

  if (! dir.empty ()) {
    QDir fi (dir.c_str ());
    m_dir = fi.absolutePath ();
  }

  QStringList files = QFileDialog::getOpenFileNames (lay::MainWindow::instance (), QString (tl::translate ("Open ").c_str ()) + m_title, m_dir.absolutePath (), m_filters);

  if (! files.isEmpty ()) {
    fp.clear ();
    for (QStringList::iterator f = files.begin (); f != files.end (); ++f) {
      if (s_use_utf8) {
        fp.push_back (f->toUtf8 ().constData ());
      } else {
        fp.push_back (f->ascii ());
      }
      QFileInfo fi (*f);
      m_dir = fi.absoluteDir ();
    }
    return true;
  } else {
    return false;
  }
#else
  if (! dir.empty ()) {
    QDir fi (dir.c_str ());
    m_dir = fi.absolutePath ();
  }

  QStringList file_names;
  for (std::vector<std::string>::const_iterator f = fp.begin (); f != fp.end (); ++f) {
    QFileInfo fi (f->c_str ());
    m_dir = fi.absoluteDir ();
    file_names << QString (f->c_str ());
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (true);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::ExistingFiles);
  fdia.setAcceptMode (QFileDialog::AcceptOpen);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Load ").c_str ()) + m_title);

  for (QStringList::iterator f = file_names.begin (); f != file_names.end (); ++f) {
    fdia.selectFile (*f);
  }

  QStringList files;
  if (fdia.exec ()) {

    files = fdia.selectedFiles();
    if (! files.isEmpty ()) {

      fp.clear ();
      for (QStringList::iterator f = files.begin (); f != files.end (); ++f) {
        if (s_use_utf8) {
          fp.push_back (f->toUtf8 ().constData ());
        } else {
          fp.push_back (f->ascii ());
        }
        QFileInfo fi (*f);
        m_dir = fi.absoluteDir ();
      }

      return true;

    }

  }

  return false;
#endif
}

bool 
FileDialog::get_save (std::string &fp) 
{
#if 1
  //  Use the standard (system) dialogs:

  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fp.c_str ();
  } else {
    file_name = m_dir.absolutePath ();
  }

  QString f = QFileDialog::getSaveFileName (lay::MainWindow::instance (), QString (tl::translate ("Save ").c_str ()) + m_title, file_name, m_filters);

  if (! f.isEmpty ()) {
    if (s_use_utf8) {
      fp = f.toUtf8 ().constData ();
    } else {
      fp = f.ascii ();
    }
    QFileInfo fi (f);
    m_dir = fi.absoluteDir ();
    return true;
  } else {
    return false;
  }
#else
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (false);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::AnyFile);
  fdia.setAcceptMode (QFileDialog::AcceptSave);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Save ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      if (s_use_utf8) {
        fp = files [0].toUtf8 ().constData ();
      } else {
        fp = files [0].ascii ();
      }
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
#endif
}

} // namespace lay

==> klayout-0.23.10.tar.gz: klayout-0.23.10/src/layFileDialog.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2015 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtGui/QFileDialog>
#include <QtCore/QFileInfo>

#include "layFileDialog.h"
#include "layMainWindow.h"
#include "tlInternational.h"

namespace lay
{

FileDialog::FileDialog (QWidget *parent, const std::string &title, const std::string &filters, const std::string &def_suffix)
  : QObject (parent)
{
  m_dir = QDir::current ();
  m_title = tl::to_qstring (title);
  m_filters = tl::to_qstring (filters);
  m_def_suffix = tl::to_qstring (def_suffix);
}

FileDialog::~FileDialog ()
{
  //  .. nothing yet ..
}

//  not used if standard dialogs are used (disabled to avoid compiler warning):
#if 0
static void
split_filters (const QString &filters, QStringList &slist)
{
  QString s;
  for (const char *cp = filters.ascii (); *cp; ++cp) {
    if (cp[0] == ';' && cp[1] == ';') {
      slist << s;
      ++cp;
      s = "";
    } else {
      s += *cp;
    }
  }

  if (s != "") {
    slist << s;
  }
}
#endif

bool 
FileDialog::get_open (std::string &fp) 
{
#if 1
  //  Use the standard (system) dialogs:

  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (tl::to_qstring (fp));
    m_dir = fi.absoluteDir ();
    file_name = tl::to_qstring (fp);
  } else {
    file_name = m_dir.absolutePath ();
  }

  QString f = QFileDialog::getOpenFileName (lay::MainWindow::instance (), tl::to_qstring (tl::translate ("Load ")) + m_title, file_name, m_filters, &m_sel_filter);

  if (! f.isEmpty ()) {
    fp = tl::to_string (f);
    QFileInfo fi (f);
    m_dir = fi.absoluteDir ();
    return true;
  } else {
    return false;
  }
#else
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (true);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::ExistingFile);
  fdia.setAcceptMode (QFileDialog::AcceptOpen);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Open ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      fp = tl::to_string (files [0]);
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
#endif
}

bool 
FileDialog::get_open (std::vector<std::string> &fp, const std::string &dir) 
{
#if 1
  //  Use the standard (system) dialogs:

  if (! dir.empty ()) {
    QDir fi (tl::to_qstring (dir));
    m_dir = fi.absolutePath ();
  }

  QStringList files = QFileDialog::getOpenFileNames (lay::MainWindow::instance (), tl::to_qstring (tl::translate ("Open ")) + m_title, m_dir.absolutePath (), m_filters, &m_sel_filter);

  if (! files.isEmpty ()) {
    fp.clear ();
    for (QStringList::iterator f = files.begin (); f != files.end (); ++f) {
      fp.push_back (tl::to_string (*f));
      QFileInfo fi (*f);
      m_dir = fi.absoluteDir ();
    }
    return true;
  } else {
    return false;
  }
#else
  if (! dir.empty ()) {
    QDir fi (dir.c_str ());
    m_dir = fi.absolutePath ();
  }

  QStringList file_names;
  for (std::vector<std::string>::const_iterator f = fp.begin (); f != fp.end (); ++f) {
    QFileInfo fi (f->c_str ());
    m_dir = fi.absoluteDir ();
    file_names << QString (f->c_str ());
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (true);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::ExistingFiles);
  fdia.setAcceptMode (QFileDialog::AcceptOpen);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Load ").c_str ()) + m_title);

  for (QStringList::iterator f = file_names.begin (); f != file_names.end (); ++f) {
    fdia.selectFile (*f);
  }

  QStringList files;
  if (fdia.exec ()) {

    files = fdia.selectedFiles();
    if (! files.isEmpty ()) {

      fp.clear ();
      for (QStringList::iterator f = files.begin (); f != files.end (); ++f) {
        fp.push_back (tl::to_string (*f));
        QFileInfo fi (*f);
        m_dir = fi.absoluteDir ();
      }

      return true;

    }

  }

  return false;
#endif
}

bool 
FileDialog::get_save (std::string &fp) 
{
#if 1
  //  Use the standard (system) dialogs:

  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (tl::to_qstring (fp));
    m_dir = fi.absoluteDir ();
    file_name = tl::to_qstring (fp);
  } else {
    file_name = m_dir.absolutePath ();
  }

  QString f = QFileDialog::getSaveFileName (lay::MainWindow::instance (), tl::to_qstring (tl::translate ("Save ")) + m_title, file_name, m_filters, &m_sel_filter);

  if (! f.isEmpty ()) {
    fp = tl::to_string (f);
    QFileInfo fi (f);
    m_dir = fi.absoluteDir ();
    return true;
  } else {
    return false;
  }
#else
  QString file_name;
  if (! fp.empty ()) {
    QFileInfo fi (fp.c_str ());
    m_dir = fi.absoluteDir ();
    file_name = fi.fileName ();
  }

  QFileDialog fdia (&lay::MainWindow::instance ());

  fdia.setDirectory (m_dir);
  if (m_def_suffix != "") {
    fdia.setDefaultSuffix (m_def_suffix);
  }

  QStringList types;
  split_filters (m_filters, types);
  fdia.setFilters (types);

  fdia.setReadOnly (false);
  fdia.setViewMode (QFileDialog::Detail);
  fdia.setFileMode (QFileDialog::AnyFile);
  fdia.setAcceptMode (QFileDialog::AcceptSave);
  fdia.setConfirmOverwrite (true);
  fdia.setCaption (QString (tl::translate ("Save ").c_str ()) + m_title);
  if (! file_name.isEmpty ()) {
    fdia.selectFile (file_name);
  }

  QStringList files;
  if (fdia.exec ()) {
    files = fdia.selectedFiles();
    if (files.size () > 0) {
      fp = tl::to_string (files [0]);
      QFileInfo fi (files [0]);
      m_dir = fi.absoluteDir ();
      return true;
    }
  }

  return false;
#endif
}

} // namespace lay
